@@ -77,10 +77,16 @@
 #define  ADJUST(min, max, a)  { if((a) < (min)) (min) = (a); if((a) > (max)) (max) = (a); }
 
 #define  INSIDE(base, a, lenght)  (((base) <= (a)) && ((a) < (base+lenght)))
+#define  INSIDE_ABS(min, a, max)  (((min) <= (a)) && ((a) <= (max)))
 
 #define  TPOS_MIN       (-INT_MAX)
 #define  TPOS_MAX       INT_MAX
 
+#define  SCALE_FACTOR   2
+#define  SPRITE_SCREEN  0
+
+#define  BITMAP_FORMAT  ".png"
+
 typedef int             tpos;
 typedef unsigned int    tcolor;
 typedef int             tcount;
@@ -100,6 +106,17 @@ public:
 
 public:
 
+  void norm(void)
+  {
+    if(r < 0) r = 0;
+    if(g < 0) g = 0;
+    if(b < 0) b = 0;
+  
+    if(r > 255) r = 255;
+    if(g > 255) g = 255;
+    if(b > 255) b = 255;
+  }
+
   rgb(void) {};
   
   rgb(int r_, int g_, int b_)
@@ -117,10 +134,16 @@ void rect_adjust(RECT *p_r, RECT *p_s);
 // -------------------------------------------------------
 //   the surface class
 // -------------------------------------------------------
+typedef enum {
+
+  BLEND_SET = 0,
+  BLEND_ADD,
+  BLEND_SUB,
+
+} BLEND_OP;
 
 typedef class surface
 {
-
   static char graphics_dir[MAX_FILENAME];
 
 public:
@@ -145,27 +168,35 @@ public:
   surface(void);
   surface(char *p_file);  
   surface(SDL_Surface *p_surf_, int used_ = 0);
-  surface(class surface *p_src, bool deep_copy = false);
-  surface(class surface &src, bool deep_copy = false);
+  surface(class surface *p_src);
+  surface(class surface &src);
+  surface(tpos width, tpos height, bool display_format = true);
+  surface(class surface *p_src, int scale = 1, bool display_format = true);
+  surface(class surface *p_src, tpos sx, tpos sy, tpos width, tpos height);
   ~surface(void);
 
-  
+  void create(tpos width, tpos height, bool display_format = true);
   void load(char *p_file);
-  void copy(class surface *p_src, bool deep_copy = false);
+  void copy(class surface *p_src, RECT *p_src_rect = NULL);
   void free(void);
 
-  
   void ckey_set(trgbcomp r, trgbcomp g, trgbcomp b);
   void ckey_set(tcolor color);
   void fill(tcolor color);
   void fill(tpos x, tpos y, tpos dx, tpos dy, tcolor color);
   void blit(class surface *p_dst, tpos tx, tpos ty);
   void blit(tpos sx, tpos sy, tpos dx, tpos dy, class surface *p_dst, tpos tx, tpos ty);
+  void blend(tpos sx, tpos sy, tpos dx, tpos dy, tcolor color, BLEND_OP operation);
 
   tcolor color_map(Uint8 r, Uint8 g, Uint8 b)
   {    
     return(SDL_MapRGB(p_surf->format, r, g, b));
   }
+
+  void color_unmap(tcolor color, Uint8 *r, Uint8 *g, Uint8 *b)
+  {    
+    SDL_GetRGB(color, p_surf->format, r, g, b);    
+  }
   
   bool is_loaded(void)
   {
@@ -191,19 +222,18 @@ public:
     *p_dy = p_surf->h;
   }
 
-
   SDL_Surface * surf_get(void)
   {
     return(p_surf);
   }
 
-  tpos surf_get_dx(void)
+  tpos width_get(void)
   {
     assert(p_surf);
     return(p_surf->w);
   }
 
-  tpos surf_get_dy(void)
+  tpos height_get(void)
   {
     assert(p_surf);
     return(p_surf->h);
@@ -217,10 +247,17 @@ public:
   RECT rect_get(void)
   {
     assert(p_surf);
-    RECT r = {0,0,p_surf->w,p_surf->h};
+    RECT r = {0,0,(Uint16)p_surf->w,(Uint16)p_surf->h};
     return(r);
   }
 
+  // Double-size from the source surface
+  void scale(class surface *p_src, tpos src_x, tpos src_y,
+             tpos width, tpos height, tpos dst_x, tpos dst_y);
+
+  // Swithches content of this and given surface
+  void content_switch(class surface *p_new);
+  
 } SURFACE;
 
 // -------------------------------------------------------
@@ -315,6 +352,14 @@ public:
     return(&rec);
   }
 
+  void rect_set(RECT *r)
+  {
+    rec = *r;
+  }
+
+  void rect_clamp(RECT *p_rect);
+  void rect_check(void);
+
 } SPRITE;
 
 // -------------------------------------------------------
@@ -360,16 +405,16 @@ public:
     return(surface_last++);
   }
 
-  surf_handle surface_copy(SURFACE *p_surf)
+  surf_handle surface_copy(SURFACE *p_surf, RECT *p_src_rec = NULL)
   {
     assert(surface_last < surface_num);
-    p_surfaces[surface_last].copy(p_surf);
+    p_surfaces[surface_last].copy(p_surf, p_src_rec);
     return(surface_last++);
   }
 
-  surf_handle surface_copy(surf_handle src_handle)
+  surf_handle surface_copy(surf_handle src_handle, RECT *p_src_rec = NULL)
   {
-    return(surface_copy(p_surfaces + src_handle));
+    return(surface_copy(p_surfaces + src_handle, p_src_rec));
   }
 
   SURFACE * surface_get(surf_handle handle)
@@ -377,13 +422,19 @@ public:
     return(p_surfaces+handle);
   }
 
+  // Switch content of the given surface with the new one
+  SURFACE * surface_switch(surf_handle handle, SURFACE *p_new)
+  {
+    p_surfaces[handle].content_switch(p_new);
+    return(p_surfaces+handle);
+  }
 
   /********************************************************
     Sprite interface
     *******************************************************
   */
 
-  spr_handle sprite_insert(const char *p_file, spr_handle first, spr_handle num = 1, spr_handle *p_last = NULL);
+  spr_handle sprite_insert(const char *p_file, spr_handle first, spr_handle *p_last = NULL);
   spr_handle sprite_insert(surf_handle sf_handle, tflag flag_ = 0, RECT *p_rect = NULL, spr_handle first = INSERT_APPEND);
   spr_handle sprite_insert(SURFACE *p_surf, tflag flag_ = 0, RECT *p_rect = NULL, spr_handle first = INSERT_APPEND);
   spr_handle sprite_insert(SPRITE *p_spr, int num = 1, spr_handle first = INSERT_APPEND);
@@ -433,14 +484,13 @@ public:
 //   the graph_2d store class
 // -------------------------------------------------------
 
-#define SURFACES 100
-#define SPRITES  10000
+#define SURFACES 1000
+#define SPRITES  100000
 #define RECT_NUM      (2*800)
 #define RECT_NUM_ALERT 800
 
 typedef class graph_2d
 {
-
   int          redraw;
 
   SURFACE     *p_screen_surface;
@@ -454,6 +504,12 @@ typedef class graph_2d
   int          rect_last;
   bool         rect_whole;
 
+  tpos         graphics_width;
+  tpos         graphics_height;
+  int          graphics_bpp;
+  bool         graphics_fullscreen;
+  int          sdl_video_flags;
+
 public:
   
   void graphics_dir_set(char *p_dir)
@@ -500,14 +556,14 @@ public:
     return((p_screen->get_height() - store.sprite_get_height(spr)) / 2);
   }
 
-  spr_handle sprite_insert(const char *p_file, spr_handle first, spr_handle num = 1, spr_handle *p_last = NULL)
+  spr_handle sprite_insert(const char *p_file, spr_handle first, spr_handle *p_last = NULL)
   {
-    return(store.sprite_insert(p_file,first,num,p_last));
+    return(store.sprite_insert(p_file,first,p_last));
   }
 
-  spr_handle sprite_insert(SPRITE *p_spr, int num = 1, spr_handle first = INSERT_APPEND)
+  spr_handle sprite_insert(SPRITE *p_spr, spr_handle first = INSERT_APPEND)
   {
-    return(store.sprite_insert(p_spr,num,first));
+    return(store.sprite_insert(p_spr,first));
   }
 
   spr_handle sprite_copy(spr_handle dst_handle, spr_handle src_handle, bool surface_copy = false)
@@ -557,6 +613,14 @@ public:
     return(p_dst->color_map(r,g,b));
   }
 
+  // Draw surface to screen or another surface
+  void draw(surface *p_src, tpos tx, tpos ty, surface *p_dst = NULL)
+  {    
+    SURFACE *p_dest = p_dst ? p_dst : p_screen_surface;
+    assert(p_src && p_dest);
+    p_src->blit(p_dest, tx, ty);
+  }
+
   void draw(spr_handle spr, tpos tx, tpos ty, spr_handle dst = 0)
   {
     SPRITE *p_spr = store.sprite_get(spr);
@@ -574,11 +638,28 @@ public:
        p_spr->blit(sx, sy, dx, dy, p_dst, tx, ty);
   }
 
+  bool fullscreen_get(void)
+  {
+    return(graphics_fullscreen);
+  }
+
+  void fullscreen_toggle(void);
+
   SPRITE * screen_get(void)
   {
     return(p_screen);
   }
-  
+
+  SURFACE * screen_surface_get(void)
+  {
+    return(p_screen_surface);
+  }
+
+  void clamp_to_screen(RECT *p_rect)
+  {
+    p_screen->rect_clamp(p_rect);
+  }
+
   void redraw_set(int red_) 
   {
     redraw = red_; 
@@ -593,6 +674,9 @@ public:
   {
     assert(rect_last < RECT_NUM);
   
+    if(!dx || !dy)
+      return;
+  
     RECT *p_tmp = rects+rect_last++;
   
     p_tmp->x = x;
@@ -600,6 +684,8 @@ public:
   
     p_tmp->w = dx;
     p_tmp->h = dy;
+
+    p_screen->rect_clamp(p_tmp);
   }
 
   void redraw_add(spr_handle spr, trec x, trec y)
@@ -613,12 +699,19 @@ public:
   
     p_tmp->w = sprite_get_width(spr);
     p_tmp->h = sprite_get_height(spr);
+
+    p_screen->rect_clamp(p_tmp);
   }
 
   void redraw_add(RECT *p_rect)
   {
     assert(rect_last < RECT_NUM);
+  
+    if(!p_rect->w || !p_rect->h)
+      return;
+  
     rects[rect_last++] = *p_rect;
+    p_screen->rect_clamp(p_rect);
   }
 
   void redraw_add(void)
@@ -646,13 +739,18 @@ public:
       }
     }
     return (TRUE);
-  }  
+  }
 
-  SDL_Surface * create_screen(int flag, int width, int height, int bpp, int fullscreen);
+  void screen_create(int flag, int width, int height, int bpp, int fullscreen);
+  void screen_destroy(void);
+  void screen_resize(int width, int height);
+  bool screen_regenerate(void);
 
   void check(void);
 
-  graph_2d(tpos dx, tpos dy, int depth, bool fullscreen) : store(SURFACES, SPRITES), rect_last(0)
+  graph_2d(tpos dx, tpos dy, int depth, bool fullscreen) 
+    : store(SURFACES, SPRITES), 
+      rect_last(0)
   {
     /* sdl init */
     bprintf("SDL Init...");
@@ -660,7 +758,7 @@ public:
 
      /* Create new screen */
     bprintf("Seting up screen %dx%d, color depth %d bits, fullscreen = %d...",dx, dy, depth, fullscreen);
-    create_screen(0, dx, dy, depth, fullscreen);
+    screen_create(0, dx, dy, depth, fullscreen);
   }
 
   ~graph_2d(void)
@@ -690,27 +788,13 @@ public:
 
 } FONT_LOOKUP_TABLE;
 
-/*
-#define FALIGN_LEFT             0
-#define FALIGN_CENTER           1
-#define FALIGN_RIGHT            2
+typedef enum _MENU_TYPE {
 
-#define FALIGN_LEFT_NO_ARROW    3
-#define FALIGN_CENTER_NO_ARROW  4
-#define FALIGN_RIGHT_NO_ARROW   5
-*/
+  MENU_LEFT,
+  MENU_CENTER,
+  MENU_RIGHT,
 
-typedef enum _ALIGNMENT {
-
-  LEFT,
-  CENTER,
-  RIGHT,
-
-  LEFT_NO_ARROW,
-  CENTER_NO_ARROW,
-  RIGHT_NO_ARROW
-
-} ALIGNMENT;
+} MENU_TYPE;
 
 #define DEFAULT_NEW_LINE_SHIFT  4
 
@@ -784,10 +868,11 @@ public:
     return extra_height;
   }
 
-  tpos print(int c, tpos px, tpos py)
-  {
+  tpos print(int c, tpos px, tpos py, bool draw = TRUE)
+  {  
     spr_handle spr = font_sprite_first+ftable.lookup(c);
-    p_grf->draw(spr, px, py);
+    if(draw)
+      p_grf->draw(spr, px, py);
     return(p_grf->sprite_get_width(spr));
   }
 
@@ -811,13 +896,20 @@ typedef class font {
   int               font_selected;
   FONT_INFO         finfo[FONT_NUM];
 
-  ALIGNMENT         align;
+  MENU_TYPE         align;
 
   // Start x,y
   tpos              sx,sy;
   // Actuall x,y
   tpos              ax,ay;
 
+  // offset for print functions
+  tpos              offset_x;
+  tpos              offset_y;
+
+  // Don't print anything, just return requested rectangle
+  bool              try_run;
+
 public:
 
   font(GRAPH_2D *p_graph);
@@ -873,8 +965,13 @@ public:
   {
     ax = sx = x; ay = sy = y;
   }
+
+  void offset_set(tpos x, tpos y)
+  {
+    offset_x = x; offset_y = y;
+  }
   
-  void alignment_set(ALIGNMENT al)
+  void alignment_set(MENU_TYPE al)
   {
     align = al;  
   }
@@ -900,7 +997,13 @@ public:
     finfo[font_index].free();
   }
 
+  void try_run_set(bool try_)
+  {
+    try_run = try_;
+  }
+
   void print(char *p_string, RECT *p_res = NULL, int lines = 0);
+  void print(RECT *p_res, const char *p_text,...);
   void print(RECT *p_res, tpos x, tpos y, const char *p_text,...);
   void print(RECT *p_res, tpos x, tpos y, int lines, const char *p_text,...);  
 
