@@ -30,25 +30,31 @@
 
 /* Basic constants
 */
-#define GAME_FPS                  30  // game refresh
+#define GAME_FPS                  (berusky_config::game_fps)  // game refresh
 
-#define GAME_RESOLUTION_X         640
-#define GAME_RESOLUTION_Y         480
+#define FULLSCREEN                (berusky_config::fullscreen)
 
-#define LEVEL_RESOLUTION_X        640
-#define LEVEL_RESOLUTION_Y        420
+#define SCREEN_DEPTH              (berusky_config::screen_depth)
+#define SCREEN_DEPTH_DEFAULT      32
 
-#define GAME_SCREEN_START_X       0
-#define GAME_SCREEN_START_Y       40
+#define DOUBLE_SIZE               (berusky_config::double_size)
+#define DOUBLE_SIZE_QUESTION      (berusky_config::double_size_question)
 
-#define EDITOR_RESOLUTION_X       1024
-#define EDITOR_RESOLUTION_Y       768
+#define GAME_RESOLUTION_X         (berusky_config::game_resolution_x)
+#define GAME_RESOLUTION_Y         (berusky_config::game_resolution_y)
 
-#define SCREEN_DEPTH              24
+#define LEVEL_SCREEN_START_X      (berusky_config::level_screen_start_x)
+#define LEVEL_SCREEN_START_Y      (berusky_config::level_screen_start_y)
+
+#define LEVEL_RESOLUTION_X        (berusky_config::level_resolution_x)
+#define LEVEL_RESOLUTION_Y        (berusky_config::level_resolution_y)
+
+#define EDITOR_RESOLUTION_X       (berusky_config::editor_resolution_x)
+#define EDITOR_RESOLUTION_Y       (berusky_config::editor_resolution_y)
 
 #define SCREEN_TOP_PANNEL_X       0
 #define SCREEN_TOP_PANNEL_Y       0
-#define SCREEN_TOP_PANNEL_DX      GAME_RESOLUTION_X
+#define SCREEN_TOP_PANNEL_DX      (DOUBLE_SIZE ? GAME_RESOLUTION_X/2 : GAME_RESOLUTION_X)
 #define SCREEN_TOP_PANNEL_DY      40
 
 #define INSERT_FIRST              0
@@ -65,29 +71,29 @@
 #define IN_LEVEL(x,y)             ((x) >= 0 && (x) < LEVEL_CELLS_X && \
                                    (y) >= 0 && (y) < LEVEL_CELLS_Y)
 
-#define CELL_SIZE_X               20
-#define CELL_SIZE_Y               20
+#define CELL_SIZE_X               (berusky_config::cell_size_x)
+#define CELL_SIZE_Y               (berusky_config::cell_size_y)
 
-#define ITEM_SIZE_X               (CELL_SIZE_X*3)
-#define ITEM_SIZE_Y               (CELL_SIZE_Y*3)
+#define EDITOR_ITEM_SIZE_X        (berusky_config::cell_size_x*3)
+#define EDITOR_ITEM_SIZE_Y        (berusky_config::cell_size_y*3)
 
-#define EDITOR_LAYER_STATUS_X     (ITEM_SIZE_X*2)
-#define EDITOR_LAYER_STATUS_Y     (ITEM_SIZE_Y*2+5)
+#define EDITOR_LAYER_STATUS_X     (20*3*2)
+#define EDITOR_LAYER_STATUS_Y     (20*3*2)
 #define EDITOR_LAYER_STATUS_DX    (EDITOR_RESOLUTION_X-EDITOR_LAYER_STATUS_X)
-#define EDITOR_LAYER_STATUS_DY    (30)
+#define EDITOR_LAYER_STATUS_DY    (20)
 
-#define EDITOR_SCREEN_START_X     (EDITOR_LAYER_STATUS_X)
-#define EDITOR_SCREEN_START_Y     (EDITOR_LAYER_STATUS_Y+EDITOR_LAYER_STATUS_DY)
+#define EDITOR_SCREEN_START_X     (berusky_config::editor_screen_start_x)
+#define EDITOR_SCREEN_START_Y     (berusky_config::editor_screen_start_y)
 
-#define EDIT_COORD_START_X        (EDITOR_SCREEN_START_X)
-#define EDIT_COORD_START_Y        (EDITOR_SCREEN_START_Y+LEVEL_RESOLUTION_Y)
-#define EDIT_COORD_DX             (LEVEL_RESOLUTION_X)
+#define EDIT_COORD_START_X        (20*3*2)
+#define EDIT_COORD_START_Y        (EDITOR_SCREEN_START_Y+LEVEL_RESOLUTION_Y-140)
+#define EDIT_COORD_DX             (LEVEL_RESOLUTION_X/2-200)
 #define EDIT_COORD_DY             (20)
 
 #define EDIT_ITEM_START_X         (EDIT_COORD_START_X)
 #define EDIT_ITEM_START_Y         (EDIT_COORD_START_Y+EDIT_COORD_DY)
-#define EDIT_ITEM_DX              (LEVEL_RESOLUTION_X)
-#define EDIT_ITEM_DY              (20)   
+#define EDIT_ITEM_DX              (600)
+#define EDIT_ITEM_DY              (20)
 
 #define EDIT_ITEM_PICT_START_X    (EDIT_ITEM_START_X)
 #define EDIT_ITEM_PICT_START_Y    (EDIT_ITEM_START_Y+EDIT_ITEM_DY+10)
@@ -97,17 +103,11 @@
 #define EDIT_HELP_START_Y         40
 #define EDIT_HELP_DY              10
 
-#define EDIT_HELP_MOUSE_X         460
+#define EDIT_HELP_MOUSE_X         600
 
-/*
-#define ITEM_MENU_X               (EDIT_ITEM_START_X)
-#define ITEM_MENU_Y               (EDIT_ITEM_PICT_START_Y+80)
-#define ITEM_MENU_DX              (LEVEL_RESOLUTION_X)
-#define ITEM_MENU_DY              (EDITOR_RESOLUTION_Y-ITEM_MENU_Y)
-*/
-#define CONSOLE_X                 (EDITOR_LAYER_STATUS_X)
-#define CONSOLE_Y                 (EDIT_ITEM_PICT_START_Y+ITEM_SIZE_Y+5)
-#define CONSOLE_DX                (EDITOR_RESOLUTION_X-CONSOLE_X-200)
+#define CONSOLE_X                 (2*EDIT_ITEM_START_X)
+#define CONSOLE_Y                 (EDIT_ITEM_PICT_START_Y+20*3+5)
+#define CONSOLE_DX                (600)
 #define CONSOLE_LINES             2
 
 #define LEVELSET_NUM              5
@@ -121,7 +121,6 @@
 #define LEVEL_LAYERS              3
 #define ALL_LEVEL_LAYERS          4
 
-
 #define SPRITE_CLEAR              0x1
 #define SPRITE_NO_DRAW            0x2
 #define SPRITE_STATIC             0x4
@@ -133,50 +132,89 @@
 #define CYBER_SPRITES             98
 #define GAME_SPRITES              59
 #define PLAYER_SPRITES            50
-#define BACKGROUND_SPRITES        2
+#define BACKGROUND_SPRITES        7
 #define MENU_SPRITES              58
 #define TEXT_LENGHT               70000
 #define FONT_SPRITES              61
 #define FONT_STEP                 100
+#define CURSOR_SPRITES            2
 
 #define FIRST_GLOBAL_LEVEL        0
 #define FIRST_CLASSIC_LEVEL       100
 #define FIRST_CYBER_LEVEL         200
 #define FIRST_BACKGROUND          400
 #define FIRST_OTHER               410
+#define FIRST_CURSOR              590
 #define FIRST_PLAYER              600
 #define FIRST_MENU                700
 #define FIRST_LOGO                800
 #define FIRST_FONT                1000
+#define FIRST_BOX_BRIGHT          2100
+#define FIRST_BOX_DARK            2150
+#define FIRST_BOX_PAPER           2200
+#define FIRST_BOX_SNOW            2250
+#define FIRST_LIGHT_BOX           2450
+#define FIRST_TNT_BRIGHT          2500
+#define FIRST_TNT_DARK            2550
+#define FIRST_TNT_PAPER           2600
+#define FIRST_TNT_SNOW            2650
+#define FIRST_TNT_SWAMP           2700
+#define FIRST_WALL_IRON_BLUE      2750
+#define FIRST_WALL_IRON_BROWN     2800
+#define FIRST_WALL_IRON_DARK      2850
+#define FIRST_WALL_IRON_GRAY      2900
+#define FIRST_WALL_MACHINE        2950
+#define FIRST_WALL_REPRO          3000
+#define FIRST_WALL_SNOW           3050
+#define FIRST_WALL_SWAMP          3100
+#define FIRST_WALL_WOOD           3150
+#define SPRITE_BLACK              3200
+#define FIRST_FLOOR_DANGER_SRC    3250
+#define FIRST_FLOOR_DANGER        3260
+#define FIRST_FLOOR_ELEVATORS_SRC 3300
+#define FIRST_FLOOR_ELEVATORS     3310
+#define FIRST_FLOOR_GRAY_SRC      3350
+#define FIRST_FLOOR_GRAY          3360
+#define FIRST_FLOOR_IRON          3400
+#define FIRST_FLOOR_SNOW          3500
 
-#define ROT_SHIFT                 2000
+#define ROT_SHIFT                 10000
 
 #define MENU_SPRIT_ROCK           FIRST_MENU
 #define MENU_SPRIT_END            (FIRST_MENU+1)
 #define MENU_SPRIT_LOGO           (FIRST_MENU+2)
-#define MENU_SPRIT_BACK           (FIRST_MENU+3)
-#define MENU_SPRIT_LOGO_SMALL_1   (FIRST_MENU+4)
-#define MENU_SPRIT_LOGO_SMALL_2   (FIRST_MENU+5)
-#define MENU_SPRIT_LOGO_SMALL_3   (FIRST_MENU+6)
+#define MENU_SPRIT_WALL           (FIRST_MENU+3)
+#define MENU_SPRIT_BACK           (FIRST_MENU+4)
+#define MENU_SPRIT_BACK1          (FIRST_MENU+5)
+#define MENU_SPRIT_BACK2          (FIRST_MENU+6)
+#define MENU_SPRIT_BACK3          (FIRST_MENU+7)
+#define MENU_SPRIT_LOGO_SMALL_1   (FIRST_MENU+8)
+#define MENU_SPRIT_LOGO_SMALL_2   (FIRST_MENU+9)
+#define MENU_SPRIT_LOGO_SMALL_3   (FIRST_MENU+10)
 #define MENU_SPRIT_LOGO_SMALL_NUM 3
 
 #define MENU_SPRIT_ARROWS         (MENU_SPRIT_LOGO_SMALL_1+MENU_SPRIT_LOGO_SMALL_NUM)
 #define MENU_ARROWS_NUM           4
 
-#define MENU_SPRIT_START          (FIRST_MENU+20)
-
 #define MENU_SPRIT_ARROW_L        (MENU_SPRIT_ARROWS)
 #define MENU_SPRIT_ARROW_LC       (MENU_SPRIT_ARROWS+1)
 #define MENU_SPRIT_ARROW_R        (MENU_SPRIT_ARROWS+2)
 #define MENU_SPRIT_ARROW_RC       (MENU_SPRIT_ARROWS+3)
 
+#define MENU_CHECKBOX_CHECKED     (MENU_SPRIT_ARROWS+MENU_ARROWS_NUM)
+#define MENU_CHECKBOX_UNCHECKED   (MENU_CHECKBOX_CHECKED+1)
+#define MENU_CHECKBOX_NUM         2
+#define MENU_SLIDEBAR             (MENU_CHECKBOX_CHECKED+2)
+#define MENU_SLIDER               (MENU_CHECKBOX_CHECKED+3)
+
+#define MENU_SPRIT_START          (FIRST_MENU+50)
 #define ANIM_PLAYER_SPRITES       10
 
 #define FIRST_KEY                 (FIRST_OTHER+4)
-#define X_KEYS_POSITION           550
+#define X_KEYS_POSITION           (550)
 #define Y_KEYS_POSITION           0
 
-#define PLAYER_MARK               (FIRST_OTHER)
+#define PLAYER_MARK               (FIRST_CURSOR)
 #define EDIT_ZEME                 (FIRST_OTHER+10)
 #define UKAZATEL_3                (FIRST_OTHER+15)
 
@@ -189,8 +227,8 @@
 #define EDITOR_MARK_RED           (FIRST_OTHER+101)
 #define EDITOR_MARK_YELLOW        (FIRST_OTHER+102)
 
-#define EDIT_ARROW_DX             20
-#define EDIT_ARROW_DY             20
+//#define EDIT_ARROW_DX             20
+//#define EDIT_ARROW_DY             20
 
 #define ANAK_LOGO                 (FIRST_LOGO)
 
@@ -203,8 +241,10 @@
 #define NO_FLOOR                  0xffff
 #define NO_ROTATION               (-1)
 
-#define GAME_ITEMS                72
+#define GAME_ITEMS                73
 
+#define COLOR_KEY_BLACK_FULL      RGB(0,0,0)
+#define COLOR_KEY_BLACK           RGB(3,3,3)
 #define COLOR_KEY_GAME            RGB(165,106,106)
 #define COLOR_KEY_MENU            RGB(255,255,255)
 
@@ -305,6 +345,8 @@
 #define PP_TOP_JAMB_Z             70
 #define PP_BOTTOM_JAMB_Z          71
 
+#define P_BOX_LIGHT               72
+
 #define DOOR_VARIATION_CLASSIC    0
 #define DOOR_VARIATION_CYBER      1
 
@@ -389,7 +431,6 @@
 #define ANIM_QUEUE_FIRST            1
 #define ANIM_QUEUE_LENGHT           100
 
-
 // Basic types
 typedef int item_handle;
 
@@ -405,6 +446,7 @@ typedef int anim_handle;
 #ifdef LINUX
 #define INI_USER_DIRECTORY "~/.berusky"
 #define INI_USER_LEVELS    "~/.berusky/User"
+#define INI_USER_PROFILES  "~/.berusky/Profiles"
 #define INI_FILE_GLOBAL    "/var/games/berusky/"INI_FILE_NAME
 #define INI_FILE_USER      "~/.berusky/"INI_FILE_NAME
 #define INI_FILE_LOCAL     "./"INI_FILE_NAME
