@@ -39,12 +39,7 @@ typedef enum {
 
 #define NO_SELECTION (-1)
 
-typedef class editor_selection {
-
-  // The current level
-  LEVEL_EDITOR *p_level;
-
-public:
+typedef struct editor_selection {
 
   // current selection
   int item;
@@ -53,8 +48,7 @@ public:
 
 public:
 
-  editor_selection(LEVEL_EDITOR *p_level_)
-  : p_level(p_level_)
+  editor_selection(void)
   {
     item = 0;
     variant = 0;
@@ -78,48 +72,67 @@ typedef enum {
 #define ITEMS_DY        (EDITOR_RESOLUTION_Y-ITEMS_Y_START)
 
 //-- finish the pannel
-#define ITEMS_IN_PANEL  12
+#define ITEMS_IN_PANEL  7
 #define ITEMS_START     0xffff
 #define ITEMS_END       0xfffe
 
+class editor_panel;
+typedef class editor_panel EDITOR_PANEL;
+
+typedef struct editor_panel_slot {  
+
+  static ITEM_REPOSITORY *p_repo;
+  
+  static void set_up(ITEM_REPOSITORY *p_repo_)
+  {    
+    p_repo = p_repo_;
+  }
+
+public:
+
+  int item;
+  int variant;
+
+public:
+
+  void draw(tpos x, tpos y, bool highlighted, bool selected);
+
+} EDITOR_PANEL_SLOT;
+
 typedef class editor_panel {
 
   // Attached panel with items
   class editor_panel *p_attached_panel;
 
   // Location and appearance of the panel
-  tpos start_x, dx;
-  tpos start_y, dy;
+  tpos                start_x, dx;
+  tpos                start_y, dy;
 
-  // Number of visible cells
-  int panel_slot_num;
-  int panel_slot_highlighted;
-  int panel_slot_selected;
-
-  int panel_handle;
+  int                 panel_handle;
 
+  MOUSE_EVENT        *p_registered_events;
+  
 protected:
+  
+  // Content of te panel
+  EDITOR_PANEL_SLOT  *p_slots;
+  int                 slot_num;
 
-  static ITEM_REPOSITORY *p_repo;
+  // Current slot configuration
+  // it's from 0 to slot_num
+  int                 visible_slot_first;
+  int                 visible_slot_num;
+
+  int                 visible_slot_highlighted;
+  int                 visible_slot_selected;
 
 public:
 
-  void attached_panel_set(class editor_panel *p_panel)
-  {
-    p_attached_panel = p_panel;
-  }
+  // Remove all items from the panel
+  void clear(void);
 
-  class editor_panel * attached_panel_get(void)
-  {
-    return(p_attached_panel);
-  }
-
-  /* Return size of panel tool (number of slots which can be displayed)
-  */
-  int panel_size_get(void)
-  {
-    return(panel_slot_num);
-  }
+  // Configure the panel
+  virtual void configure(int item) {};
 
   /* Return index of slot at x,y coordinates
   */
@@ -134,25 +147,27 @@ public:
     return(slot_return(x, y, slot) ? slot : NO_SELECTION);
   }
 
-  /* Draw one slot of the panel
-  */
-  void slot_draw(int slot, int item, int variant);
+  bool slot_valid(int slot)
+  {
+    return(slot >= 0 && slot < slot_num);
+  }
+
+  EDITOR_PANEL_SLOT * slot_get(int slot)
+  {
+    return(slot_valid(slot) ? p_slots+slot : NULL);
+  }
 
-  /* Draw panel controls
-  */
-  void controls_draw(void);
-  
   /* Panel highlight routines
   */
   void slot_highlight(int slot, bool redraw)
   {
-    panel_slot_highlighted = slot;
+    visible_slot_highlighted = slot;
   
-    assert(panel_slot_highlighted >= NO_SELECTION && 
-           panel_slot_highlighted < panel_slot_num);
+    assert(visible_slot_highlighted >= NO_SELECTION && 
+           visible_slot_highlighted < slot_num);
   
     if(redraw)
-      panel_draw();
+      panel_draw(redraw);
   }
   void slot_highlight(tpos x, tpos y, bool redraw)
   {
@@ -168,20 +183,28 @@ public:
   void slot_select(int slot, EDITOR_SELECTION *p_sel, bool propagate, bool redraw)
   {
     // Set selected slot
-    panel_slot_selected = slot;
+    visible_slot_selected = slot;
   
-    assert(panel_slot_selected >= NO_SELECTION && 
-           panel_slot_selected < panel_slot_num);
+    assert(visible_slot_selected >= NO_SELECTION && 
+           visible_slot_selected < slot_num);
   
     if(redraw) {
-      panel_draw();
+      panel_draw(redraw);
     }
   
-    // Propagate to editor selection (item/variant)
+    // Propagate to editor selection (item/variant)  
     if(propagate) {
-      item_select(p_sel);
-    }  
+      EDITOR_SELECTION selection;
+      slot_selection_get(&selection);
+      if(p_attached_panel && slot != NO_SELECTION) {
+        p_attached_panel->configure(selection.item);
+        p_attached_panel->panel_draw(redraw);
+      }
+      if(p_sel)
+        *p_sel = selection;
+    }
   }
+
   void slot_select(tpos x, tpos y, EDITOR_SELECTION *p_sel, bool propagate, bool redraw)
   {
     slot_select(slot_return(x,y),p_sel,propagate,redraw);
@@ -193,62 +216,89 @@ public:
 
   void slot_selection_fix(int last_valid_slot, bool redraw)
   {
-    if(panel_slot_selected != NO_SELECTION) {
-      if(panel_slot_selected > last_valid_slot)
-        panel_slot_selected = last_valid_slot;
+    if(visible_slot_selected != NO_SELECTION) {
+      if(visible_slot_selected >= slot_num)
+        visible_slot_selected = slot_num-1;
     }
-    if(panel_slot_highlighted != NO_SELECTION) {
-      if(panel_slot_highlighted > last_valid_slot)
-        panel_slot_highlighted = last_valid_slot;
+    if(visible_slot_highlighted != NO_SELECTION) {
+      if(visible_slot_highlighted >= slot_num)
+        visible_slot_highlighted = slot_num-1;
     }
     if(redraw) {
-      panel_draw();
+      panel_draw(redraw);
     }
   }
 
-  int selected_slot_get(void)
-  {
-    return(panel_slot_selected);
-  }  
+  void slot_selection_get(EDITOR_SELECTION *p_sel)
+  {    
+    if(p_sel && visible_slot_selected != NO_SELECTION) {
+      p_sel->item = p_slots[visible_slot_selected].item;
+      p_sel->variant = p_slots[visible_slot_selected].variant;
+    }
+  }
 
-  /* Is the panel slot valid?
-     It's a relative slot positition, from 0 to panel_item_num
-  */
-  virtual bool slot_valid(int slot) = 0;
+protected:
 
-  /* Return first item on the panel
-  */
-  virtual int item_firts_get(void) = 0;
+  static ITEM_REPOSITORY *p_repo;
+
+public:
   
-  /* Return last valid item (0 is the first one)
-  */
-  virtual int items_num_get(void) = 0;
-  
-  /* Select an editor item. It's based on recent selected slot
-  */
-  virtual void item_select(EDITOR_SELECTION *p_sel) = 0;
-
-  /* Set an item for first slot
-  */
-  virtual void panel_set(int first, EDITOR_SELECTION *p_sel, bool propagate, bool redraw) = 0;
-
-  /* Draws controls and panel item(s)
-  */
-  virtual void panel_draw(void) = 0;
-
-  /* Scroll the pannel with given direction
-     and preserve selected item
-  */
-  void panel_scroll(int direction, EDITOR_SELECTION *p_sel, bool redraw);
-
   /* Setting up
   */
   static void set_up(ITEM_REPOSITORY *p_repo_)
   {    
     p_repo = p_repo_;
   }
+
+public:
+
+  static int item_base_get(int item);
+
+  void attached_panel_set(class editor_panel *p_panel)
+  {
+    p_attached_panel = p_panel;
+  }
+
+  class editor_panel * attached_panel_get(void)
+  {
+    return(p_attached_panel);
+  }
+
+  void panel_set(int slot_first_new, EDITOR_SELECTION *p_sel, 
+                 bool propagate, bool redraw)
+  {
+    if(slot_num <= visible_slot_num) {
+      slot_first_new = 0;
+    }
+    else if(slot_first_new + visible_slot_num > slot_num) {
+      slot_first_new = slot_num - visible_slot_num;
+      assert(slot_first_new >= visible_slot_first);
+    }
+
+    if(slot_first_new != visible_slot_first) {
+      visible_slot_first = slot_first_new;
+
+      if(slot_num < visible_slot_num) {
+        slot_selection_fix(slot_num-1, redraw);
+      }
+      else {
+        slot_selection_fix(visible_slot_num-1, redraw);
+      }
+
+      if(propagate)
+        slot_selection_get(p_sel);
+    }
+  }
+
+  // Draws controls and panel item(s)
+  void panel_draw(bool draw);
+
+  // Scroll the pannel with given direction
+  // and preserve selected item
+  void panel_scroll(int direction, EDITOR_SELECTION *p_sel, bool redraw);
   
   void register_controls_events(INPUT *p_input);
+  void unregister_controls_events(INPUT *p_input);
 
   // Get pannel size, without control buttons
   RECT boundary_get(void);
@@ -258,24 +308,24 @@ public:
   {
     switch(direction) {
       case HORIZONTAL:
-        dx = ITEM_SIZE_X;
+        dx = EDITOR_ITEM_SIZE_X;
         dy = 0;
         break;
       case VERTICAL:
         dx = 0;
-        dy = ITEM_SIZE_Y;
+        dy = EDITOR_ITEM_SIZE_Y;
         break;
     }
 
-    panel_slot_num = panel_size;
+    visible_slot_num = panel_size;
   
     start_x = sx;
     start_y = sy;
   
     panel_handle = handle;
 
-    panel_slot_highlighted = NO_SELECTION;
-    panel_slot_selected = NO_SELECTION;
+    visible_slot_highlighted = NO_SELECTION;
+    visible_slot_selected = NO_SELECTION;
   }  
 
   editor_panel(int panel_item_num, DIRECTION direction, 
@@ -288,170 +338,40 @@ public:
 
 } EDITOR_PANEL;
 
-/* Derived panel for items
-*/
-typedef class item_panel : public editor_panel {
-
-  // Items in the panel
-  int panel_item_first;
-
-public:
-  
-  /* Items in panel
-  */  
-  void panel_set(int first, EDITOR_SELECTION *p_sel, bool propagate, bool redraw)
-  {
-    panel_item_first = first;
-  
-    if(items_num_get() <= panel_size_get()) {  
-      slot_selection_fix(items_num_get()-1, redraw);
-    }
-    else {
-      slot_selection_fix(panel_size_get(), redraw);
-    }
-  
-    if(propagate)
-      item_select(p_sel);
-  }
-
-  /* Return first item on the panel
-  */
-  int item_firts_get(void)
-  {
-    return(panel_item_first);
-  }
-  
-  /* Return last valid item (0 is the first one)
-  */
-  int items_num_get(void)
-  {
-    return(p_repo->item_num_get());
-  }
-
-  /* Select the select item by variant panel
-  */
-  void item_select(EDITOR_SELECTION *p_sel);
-
-  bool slot_valid(int slot)
-  {
-    if(slot >= 0 && slot < panel_size_get()) {
-      return(item_firts_get() + slot < items_num_get());
-    }
-    return(FALSE);
-  }
-
-public:
-  
-  void panel_draw(void);
-
-public:
-
-  item_panel(int panel_item_num, DIRECTION direction,
-             tpos start_x, tpos start_y, int handle)
-  : editor_panel(panel_item_num, direction, start_x, start_y, handle),    
-    panel_item_first(0)
-  {  
-  }
-
-} ITEM_PANEL;
-
 /* Derived panel for variants
 */
 typedef class variant_panel : public editor_panel {
 
-  // Item which variants are published
-  int panel_item;
-
-  // Variants in the panel
-  int panel_variant_first;
-
 public:
   
-  void panel_item_set(int item, bool redraw)
-  {
-    panel_item = item;
-    if(redraw)
-      panel_draw();
-  }
-
-  /* Items in panel
-  */  
-  void panel_set(int first, EDITOR_SELECTION *p_sel, bool propagate, bool redraw)
-  {
-    panel_variant_first = first;
-  
-    if(variant_last_get() < panel_size_get()) {  
-      slot_selection_fix(items_num_get()-1, redraw);
-    }
-    else {
-      slot_selection_fix(panel_size_get(), redraw);
-    }  
-  
-    if(propagate)
-      item_select(p_sel);
-  }
-
-  /* Return first variant on the panel
-  */
-  int variant_firts_get(void)
-  {
-    return(panel_variant_first);
-  }
-  
-  /* Return last valid item (0 is the first one)
-  */
-  int variant_last_get(void)
-  {
-    return(p_repo->variants_get(panel_item));
-  }
-
-  /* Return first item on the panel
-  */
-  int item_firts_get(void)
-  {
-    return(variant_firts_get());
-  }
-  
-  /* Return last valid item (0 is the first one)
-  */
-  int items_num_get(void)
-  {
-    return(variant_last_get());
-  }
-
-  void item_select(EDITOR_SELECTION *p_sel)
-  {
-    int slot = selected_slot_get();
-    if(p_sel && slot != NO_SELECTION) {
-      p_sel->item = panel_item;
-      p_sel->variant = panel_variant_first+slot;
-    }
-  }
-
-  bool slot_valid(int slot)
-  {
-    if(slot >= 0 && slot < panel_size_get()) {
-      return(variant_firts_get() + slot < variant_last_get());
-    }
-    return(FALSE);
-  }
-
-public:
-  
-  void panel_draw(void);
+  // Configure variant panel according the selected item
+  virtual void configure(int item);
 
 public:
 
   variant_panel(int panel_size, DIRECTION direction,
                 tpos start_x, tpos start_y, int handle)
-  : editor_panel(panel_size, direction, start_x, start_y, handle),    
-    panel_item(0),
-    panel_variant_first(0)
-  {  
+  : editor_panel(panel_size, direction, start_x, start_y, handle)
+  {
   }
 
 } VARIANT_PANEL;
 
+/* Derived panel for items
+*/
+typedef class item_panel : public editor_panel {
+
+public:
+  
+  item_panel(int panel_item_num,
+             DIRECTION direction,
+             tpos start_x, tpos start_y, 
+             int handle, 
+             VARIANT_PANEL *p_var);
+
+} ITEM_PANEL;
+
+
 typedef class editor_layer_config {
 
 public:
@@ -500,6 +420,7 @@ typedef class editor_console {
 
   int          lines;
   char         output_lines[CONSOLE_OUTPUT_LINES][CONSOLE_MAX_INPUT_LINE];
+  bool         output_draw;
 
   tpos         ix_title;
   tpos         iy_title;
@@ -525,7 +446,8 @@ public:
   
   void print(const char *p_text,...);
   void output_clear(bool redraw = TRUE);
-  void output_redraw(void);
+  void output_redraw(bool draw);
+  void output_draw_set(bool draw);
 
 
   // ----------------------------------------------
@@ -600,7 +522,8 @@ typedef class editor_gui : public gui_base {
 
   ITEM_REPOSITORY        *p_repo;
   LEVEL_EDITOR            level;
-  bool                    draw_level; // Draw the level?
+  bool                    draw_level;   // Draw the level?
+  bool                    draw_panels;  // Draw the panels?
 
   LEVEL_EVENT_QUEUE       queue;
 
@@ -623,6 +546,10 @@ public:
   void  console_wait(MENU_STATE state, int data, int data1);
   void  input_stop(bool success);  
 
+  void  console_draw(bool draw)
+  {
+    console.output_redraw(draw);
+  }
 
   bool  input_get_bool(void)
   {
@@ -640,16 +567,16 @@ public:
   }
 
 public:
-
-  #define                 PANELS        4
-  EDITOR_PANEL           *ipanel[PANELS];
+  
+  ITEM_PANEL      panel_items;
+  VARIANT_PANEL   panel_variants;
 
 public:
   
   // Panel interface
   void panel_item_select(int panel, tpos x, tpos y);
   void panel_item_highlight(int panel, tpos x, tpos y);
-  void panel_draw(void);
+  void panel_draw(bool draw);
   void panel_scroll(int panel, int direction);
   void panel_scroll_mouse(int direction);
 
@@ -658,20 +585,29 @@ private:
   // Recent selected item (in editor)
   EDITOR_SELECTION        selected_editor_item;
 
+private:
+
+  void selection_cursor_draw_status(bool draw, char *p_text,...);
+
 public:
   
   // Selection interface
-  void selection_draw(void);
-
-  void selection_cursor_draw_status(char *p_text,...);
-  void selection_cursor_update(void);
+  void selection_draw(bool draw);
+  void selection_cursor_draw(bool draw);
 
   void selection_rotate(int direction = 1);
 
+  // Pick up selected item from cursor
+  void selection_pickup(void);
+
 private:
 
+  LLIST_ITEM * p_side_event_first;
+  int          side_event_num;
+
   void side_menu_create(void);
-  void side_menu_draw(void);
+  void side_menu_remove(void);
+  void side_menu_draw(bool draw);
 
 private:
 
@@ -705,12 +641,16 @@ public:
   void level_save_as(char *p_file = NULL, int force = FALSE);
   void level_save_as_callback(void);
 
+  void level_move(tpos dx, tpos dy);
+
+  void draw(bool force = FALSE);
+
 public:
 
   editor_gui(ITEM_REPOSITORY *p_repo_, DIR_LIST *p_dir_);
   ~editor_gui(void);
   
-  void editor_reset(void);
+  void editor_init(void);
 
   bool editor_quit(bool force);
   void editor_quit_callback(void);
@@ -719,8 +659,8 @@ public:
   void test(void);
   void test_gui(void);
 
-  // Level screen interface
-  void level_draw(void);
+  // Level screen interface  
+  bool level_draw(void);
   void level_config(void);
 
   // Menu interface
@@ -737,11 +677,13 @@ public:
   
   void level_item_clear(void);
   void level_item_clear_rectangle(bool filled);
-  
-  void layer_menu_draw(void);
-  void layer_status_draw(void);
+
+  void layer_active_draw(bool draw);
+
+  void layer_status_create(void);
+  void layer_status_draw(bool draw);
   void layer_status_switch(int layer, LAYER_STATE state);  
-  void layer_active_set(int layer);
+  void layer_active_set(int layer, bool draw);
   int  layer_active_get(void);
   int  layer_active_get(tpos x, tpos y);
   
@@ -761,6 +703,9 @@ public:
   // Run level
   void editor_run_level(void);
   
+  // Run level in fullscreen
+  void editor_fullscreen(void);
+  
   // Shade level 
   void editor_shader(void);
   void editor_shader(tpos sx, tpos sy, tpos dx, tpos dy);
@@ -782,9 +727,7 @@ public:
 #define SCREEN_HANDLE_ROTATE  3
 #define SCREEN_HANDLE_VARIATE 4
 
-#define PANEL_HANDLE_1        11
-#define PANEL_HANDLE_2        12
-#define PANEL_HANDLE_3        13
-#define PANEL_HANDLE_4        14
+#define PANEL_HANDLE_ITEMS    11
+#define PANEL_HANDLE_VARIANTS 12
 
 #endif // __EDITOR_H__
