@@ -284,7 +284,7 @@ public:
   bool coord_in_level(tpos x, tpos y)
   { 
     return(p_scr->coord_in_area(x,y));
-  }      
+  }
 
   // Translate global coordination to cell coordination
   void coord_to_cell(tpos *p_x, tpos *p_y)
@@ -299,9 +299,15 @@ public:
   }
 
   // Set offset of level window
-  void window_offset(tpos start_x, tpos start_y)
+  void window_offset_set(tpos start_x, tpos start_y)
   {
-    p_scr->window_offset(start_x, start_y);
+    p_scr->window_offset_set(start_x, start_y);
+  }
+
+  // Set offset of level window
+  void window_offset_get(tpos &start_x, tpos &start_y)
+  {
+    p_scr->window_offset_get(start_x, start_y);
   }
 
  /***********************************************************************
@@ -534,7 +540,7 @@ public:
                              bool graphics_change)
   {
     level_cell *p_cell = cells[x][y]+layer;
-      
+    
     p_cell->variation += direction;
   
     int variation_max = p_repo->variants_get(p_cell->item)-1;
@@ -1058,6 +1064,13 @@ public:
 
   void level_populate(void);
 
+  void level_move(tpos dx, tpos dy);
+  
+  void level_selection_area_set(bool active)
+  {
+    ((SCREEN_EDITOR *)p_scr)->selection_area_set(active);
+  }
+
   // -------------------------------------------------------------------
   // undo/restore interface
   // -------------------------------------------------------------------
@@ -1102,6 +1115,15 @@ public:
     return(back_max);
   }
 
+public:
+  
+  bool level_draw(void);
+
+  void level_redraw(void)
+  {
+    p_scr->set_redraw();
+  }
+
 public:
   
   level_editor(ITEM_REPOSITORY *p_rep)
